from typing import Optional, List
from tqdm import tqdm
from PIL.Image import Image
from panna import SVD, SDUpScaler, InstructIR
from panna.util import get_logger

logger = get_logger(__name__)


class PipelineSVDUpscale:

    def __init__(self, upscaler: str = "diffusion"):
        self.svd = SVD()
        if upscaler == "diffusion":
            self.upscaler = SDUpScaler()
        elif upscaler == "instruct_ir":
            self.upscaler = InstructIR()
        else:
            raise ValueError(f"Unknown upscaler: {upscaler}")

    def __call__(self,
                 image: Image,
                 output_path: Optional[str] = None,
                 prompt: Optional[str] = None,
                 decode_chunk_size: Optional[int] = None,
                 num_frames: int = 25,
                 motion_bucket_id: int = 127,
                 fps: int = 7,
                 noise_aug_strength: float = 0.02,
                 seed: Optional[int] = None) -> List[Image]:
        logger.info("run image2video")
        frames = self.svd.image2video(
            [image],
            decode_chunk_size=decode_chunk_size,
            num_frames=num_frames,
            motion_bucket_id=motion_bucket_id,
            fps=fps,
            noise_aug_strength=noise_aug_strength,
            seed=seed
        )[0]
        logger.info("run image restoration")
        new_frames = []
        for frame in tqdm(frames):
            frame = self.upscaler.image2image(
                [frame],
                prompt=[prompt]
            )[0]
            new_frames.append(frame)
        if output_path:
            self.svd.export(new_frames, output_path, fps)
        return new_frames



import torch
from torch import nn
from torch.nn.functional import normalize
from transformers import AutoModel, AutoTokenizer


POOL_MODELS = {"sentence-transformers/all-MiniLM-L6-v2", "TaylorAI/bge-micro-v2"}


def mean_pooling(model_output, attention_mask):
    token_embeddings = model_output[0]
    input_mask_expanded = attention_mask.unsqueeze(-1).expand(token_embeddings.size()).float()
    return torch.sum(token_embeddings * input_mask_expanded, 1) / torch.clamp(input_mask_expanded.sum(1), min=1e-9)


class LanguageModel(nn.Module):
    def __init__(self, model='TaylorAI/bge-micro-v2'):
        super(LanguageModel, self).__init__()
        self.tokenizer = AutoTokenizer.from_pretrained(model)
        self.model = AutoModel.from_pretrained(model)
        self.model_name = model
        if "clip" in self.model_name:
            self.model.vision_model = None
        for param in self.model.parameters():
            param.requires_grad = False
        self.model.eval()

    def forward(self, text_batch):
        inputs = self.tokenizer(text_batch, padding=True, truncation=True, return_tensors="pt")
        inputs = {k: v.to(self.model.device) for k, v in inputs.items()}
        with torch.no_grad():
            outputs = self.model(**inputs)
        sentence_embeddings = mean_pooling(outputs, inputs['attention_mask'])
        sentence_embedding = normalize(sentence_embeddings, p=2, dim=1)
        return sentence_embedding
    

class LMHead(nn.Module):
    def __init__(self, embedding_dim=384, hidden_dim=256, num_classes=7):
        super(LMHead, self).__init__()
        self.fc1 = nn.Linear(embedding_dim, hidden_dim)
        self.fc2 = nn.Linear(hidden_dim, num_classes)
        
    def forward(self, x):
        embd = self.fc1(x)
        embd = normalize(embd, p=2, dim=1)
        deg_pred = self.fc2(embd)
        return embd, deg_pred



import os
from panna import SD3

model = SD3()
os.makedirs("./test/test_image", exist_ok=True)
prompts = [
    "A majestic lion jumping from a big stone at night",
    "Study for a portrait of lady reading newspaper by Francis Bacon, the British painter"
]
prompts_neg = [
    "low quality",
    "monotone"
]

for i, (p, p_n) in enumerate(zip(prompts, prompts_neg)):
    output = model.text2image([p], batch_size=1, seed=42)
    model.export(output[0], f"./test/test_images/test_sd3.{i}.png")
    output = model.text2image([p], negative_prompt=[p_n], batch_size=1, seed=42)
    model.export(output[0], f"./test/test_images/test_sd3.{i}.negative.png")
    output = model.text2image([p], negative_prompt=[p_n], batch_size=1, seed=42, width=1024, height=720)
    model.export(output[0], f"./test/test_images/test_sd3.{i}.negative.landscape.png")




from typing import List
from tqdm import tqdm
import torch
from clip_interrogator import Config, Interrogator
from PIL.Image import Image
from .util import clear_cache, get_logger

logger = get_logger(__name__)


class CLIPInterrogator:

    def __init__(self):
        config = Config()
        config.device = 'cuda' if torch.cuda.is_available() else 'cpu'
        config.blip_offload = False if torch.cuda.is_available() else True
        config.chunk_size = 2048
        config.flavor_intermediate_count = 512
        config.blip_num_beams = 64
        self.ci = Interrogator(config)

    def image2text(self, image: List[Image], best_max_flavors: int = 32):
        captions = []
        for i in tqdm(image):
            captions.append(self.ci.interrogate(i.convert('RGB'), max_flavors=best_max_flavors))
            clear_cache()
        return captions



from panna import ControlNetSD3
from diffusers.utils import load_image
from panna.util import clear_cache

prompt = 'Anime style illustration of a girl wearing a suit. A moon in sky. In the background we see a big rain approaching. text "InstantX" on image'
n_prompt = 'NSFW, nude, naked, porn, ugly'
control_image = load_image("https://huggingface.co/InstantX/SD3-Controlnet-Tile/resolve/main/tile.jpg")

# test tile
model = ControlNetSD3(condition_type="tile")
output = model.text2image([prompt], negative_prompt=[n_prompt], image=[control_image], guidance_scale=0.5)
model.export(output[0], "./test/test_image/test_controlnet_stable_diffusion_3.tile.png")
del model
clear_cache()

# test pose
model = ControlNetSD3(condition_type="pose")
control_image = load_image("https://huggingface.co/InstantX/SD3-Controlnet-Pose/resolve/main/pose.jpg")
output = model.text2image([prompt], negative_prompt=[n_prompt], image=[control_image], guidance_scale=0.5)
model.export(output[0], "./test/test_image/test_controlnet_stable_diffusion_3.pose.png")
del model
clear_cache()

# test canny
model = ControlNetSD3(condition_type="canny")
control_image = load_image("https://huggingface.co/InstantX/SD3-Controlnet-Canny/resolve/main/canny.jpg")
output = model.text2image([prompt], negative_prompt=[n_prompt], image=[control_image], guidance_scale=0.5)
model.export(output[0], "./test/test_image/test_controlnet_stable_diffusion_3.canny.png")
del model
clear_cache()



# ------------------------------------------------------------------------
# Copyright (c) 2022 megvii-model. All Rights Reserved.
# ------------------------------------------------------------------------
# Source: https://github.com/megvii-research/NAFNet

import torch
import torch.nn as nn
from .nafnet_utils import LayerNorm2d


class SimpleGate(nn.Module):
    def forward(self, x):
        x1, x2 = x.chunk(2, dim=1)
        return x1 * x2


class NAFBlock(nn.Module):
    def __init__(self, c, DW_Expand=2, FFN_Expand=2, drop_out_rate=0.):
        super().__init__()
        dw_channel = c * DW_Expand
        self.conv1 = nn.Conv2d(in_channels=c, out_channels=dw_channel, kernel_size=1, padding=0, stride=1, groups=1, bias=True)
        self.conv2 = nn.Conv2d(in_channels=dw_channel, out_channels=dw_channel, kernel_size=3, padding=1, stride=1, groups=dw_channel,
                               bias=True)
        self.conv3 = nn.Conv2d(in_channels=dw_channel // 2, out_channels=c, kernel_size=1, padding=0, stride=1, groups=1, bias=True)
        
        # Simplified Channel Attention
        self.sca = nn.Sequential(
            nn.AdaptiveAvgPool2d(1),
            nn.Conv2d(in_channels=dw_channel // 2, out_channels=dw_channel // 2, kernel_size=1, padding=0, stride=1,
                      groups=1, bias=True),
        )

        # SimpleGate
        self.sg = SimpleGate()
        ffn_channel = FFN_Expand * c
        self.conv4 = nn.Conv2d(in_channels=c, out_channels=ffn_channel, kernel_size=1, padding=0, stride=1, groups=1, bias=True)
        self.conv5 = nn.Conv2d(in_channels=ffn_channel // 2, out_channels=c, kernel_size=1, padding=0, stride=1, groups=1, bias=True)
        self.norm1 = LayerNorm2d(c)
        self.norm2 = LayerNorm2d(c)
        self.dropout1 = nn.Dropout(drop_out_rate) if drop_out_rate > 0. else nn.Identity()
        self.dropout2 = nn.Dropout(drop_out_rate) if drop_out_rate > 0. else nn.Identity()
        self.beta = nn.Parameter(torch.zeros((1, c, 1, 1)), requires_grad=True)
        self.gamma = nn.Parameter(torch.zeros((1, c, 1, 1)), requires_grad=True)

    def forward(self, inp):
        x = inp
        x = self.norm1(x)
        x = self.conv1(x)
        x = self.conv2(x)
        x = self.sg(x)
        x = x * self.sca(x)
        x = self.conv3(x)
        x = self.dropout1(x)
        y = inp + x * self.beta
        x = self.conv4(self.norm2(y))
        x = self.sg(x)
        x = self.conv5(x)
        x = self.dropout2(x)
        return y + x * self.gamma



from typing import Optional, List
import torch
from PIL.Image import Image
from diffusers import LEditsPPPipelineStableDiffusionXL, DiffusionPipeline
from panna.util import get_generator, clear_cache, get_logger

logger = get_logger(__name__)
preset_parameter = {
    "default": {
        "guidance_scale": 7,
        "warmup_step": 2,
        "threshold": 0.95
    },
    "style": {
        "guidance_scale": 7,
        "warmup_step": 2,
        "threshold": 0.5
    },
    "face": {
        "guidance_scale": 5,
        "warmup_step": 2,
        "threshold": 0.95
    },
    "object": {
        "guidance_scale": 12,
        "warmup_step": 5,
        "threshold": 0.9
    }
}


class PipelineLEditsPP:

    base_model_id: str
    base_model: LEditsPPPipelineStableDiffusionXL
    refiner_model: Optional[DiffusionPipeline] = None

    def __init__(self,
                 base_model_id: str = "stabilityai/stable-diffusion-xl-base-1.0",
                 variant: Optional[str] = "fp16",
                 torch_dtype: Optional[torch.dtype] = torch.float16,
                 device_map: str = "balanced",
                 low_cpu_mem_usage: bool = True):
        self.base_model = LEditsPPPipelineStableDiffusionXL.from_pretrained(
            base_model_id, use_safetensors=True, variant=variant, torch_dtype=torch_dtype, device_map=device_map, low_cpu_mem_usage=low_cpu_mem_usage
        )

    def __call__(self,
                 image: Image,
                 edit_prompt: List[str],
                 reverse_editing_direction: List[bool],
                 output_path: Optional[str] = None,
                 edit_guidance_scale: Optional[List[float]] = None,
                 edit_threshold: Optional[List[float]] = None,
                 edit_warmup_steps: Optional[List[int]] = None,
                 edit_style: Optional[List[str]] = None,
                 num_inversion_steps: int = 50,
                 skip: float = 0.2,
                 seed: Optional[int] = None) -> Image:
        self.base_model.vae.to(dtype=torch.float16)  # this prevents raising error of half-precision
        edit_style = ["default"] * len(edit_prompt) if edit_style is None else edit_style
        if edit_guidance_scale is None:
            edit_guidance_scale = [preset_parameter[i]["guidance_scale"] for i in edit_style]
        if edit_threshold is None:
            edit_threshold = [preset_parameter[i]["threshold"] for i in edit_style]
        if edit_warmup_steps is None:
            edit_warmup_steps = [preset_parameter[i]["warmup_step"] for i in edit_style]

        logger.info("image inversion")
        self.base_model.invert(image=image, num_inversion_steps=num_inversion_steps, skip=skip)
        clear_cache()

        logger.info("semantic guidance")
        image = self.base_model(
            image=image,
            editing_prompt=edit_prompt,
            reverse_editing_direction=reverse_editing_direction,
            edit_guidance_scale=edit_guidance_scale,
            edit_threshold=edit_threshold,
            edit_warmup_steps=edit_warmup_steps,
            generator=get_generator(seed)
        ).images[0]
        if output_path:
            image.save(output_path)
        clear_cache()
        return image



"""StableVideoDiffusion https://huggingface.co/docs/diffusers/en/using-diffusers/svd."""
from typing import Optional, List
from tqdm import tqdm
import torch
from diffusers import StableVideoDiffusionPipeline
from diffusers.utils import load_image, export_to_video
from PIL.Image import Image
from .util import get_generator, clear_cache, get_logger, resize_image

logger = get_logger(__name__)


class SVD:

    base_model: StableVideoDiffusionPipeline
    height: int = 576
    width: int = 1024

    def __init__(self,
                 base_model_id: str = "stabilityai/stable-video-diffusion-img2vid-xt-1-1",
                 variant: str = "fp16",
                 torch_dtype: torch.dtype = torch.float16,
                 device_map: str = "balanced",
                 low_cpu_mem_usage: bool = True):
        self.base_model = StableVideoDiffusionPipeline.from_pretrained(
            base_model_id, use_safetensors=True, variant=variant, torch_dtype=torch_dtype, device_map=device_map, low_cpu_mem_usage=low_cpu_mem_usage
        )

    def image2video(self,
                    image: List[Image],
                    decode_chunk_size: Optional[int] = None,
                    num_frames: int = 25,
                    motion_bucket_id: int = 127,
                    fps: int = 7,
                    noise_aug_strength: float = 0.02,
                    height: Optional[int] = None,
                    width: Optional[int] = None,
                    seed: Optional[int] = None) -> List[List[Image]]:
        """Generate video from image.

        :param image:
        :param decode_chunk_size:
        :param num_frames:
        :param motion_bucket_id: The motion bucket id to use for the generated video. This can be used to control the
            motion of the generated video. Increasing the motion bucket id increases the motion of the generated video.
        :param fps: The frames per second of the generated video.
        :param noise_aug_strength: The amount of noise added to the conditioning image. The higher the values the less
            the video resembles the conditioning image. Increasing this value also increases the motion of the
            generated video.
        :param height:
        :param width:
        :param seed:
        :return:
        """
        output_list = []
        height = self.height if height is None else height
        width = self.width if width is None else width
        for i in tqdm(image):
            output_list.append(self.base_model(
                resize_image(load_image(i), width=width, height=height),
                decode_chunk_size=decode_chunk_size,
                num_frames=num_frames,
                motion_bucket_id=motion_bucket_id,
                fps=fps,
                noise_aug_strength=noise_aug_strength,
                height=height,
                width=width,
                generator=get_generator(seed)
            ).frames[0])
        clear_cache()
        return output_list

    @staticmethod
    def export(data: List[Image], output_path: str, fps: int = 7) -> None:
        export_to_video(data, output_path, fps=fps)



import os
from panna import InstructIR
from diffusers.utils import load_image


model = InstructIR()
sample_image = load_image("https://huggingface.co/spaces/multimodalart/stable-video-diffusion/resolve/main/images/wink_meme.png")
os.makedirs("./test/test_image", exist_ok=True)

output = model.image2image([sample_image])
model.export(output[0], "./test/test_image/test_instruct_ir.png")

output = model.image2image([sample_image], prompt=["Correct the motion blur in this image so it is more clear"])
model.export(output[0], "./test/test_image/test_instruct_ir.prompt1.png")

output = model.image2image([sample_image], prompt=["please I want this image for my photo album, can you edit it as a photographer"])
model.export(output[0], "./test/test_image/test_instruct_ir.prompt2.png")



from PIL import Image
from panna import Depth2Image

model = Depth2Image()
img = Image.open("./test/sample_image.png")
output = model.text2image(
    [img],
    prompt=["a black cat"],
    negative_prompt=["bad, deformed, ugly, bad anatomy"],
    seed=42
)
model.export(output[0], "./test/test_image/test_depth2image.png")


import os
from diffusers.utils import load_image
from panna.pipeline import PipelineSVDUpscale

img = load_image("https://huggingface.co/spaces/multimodalart/stable-video-diffusion/resolve/main/images/wink_meme.png")
os.makedirs("./test/test_video", exist_ok=True)


pipe = PipelineSVDUpscale()
pipe(
    img,
    prompt="a lady, high quality, clean",
    output_path="./test/test_video/test_svd_upscale.motion_bucket_id=127.noise_aug_strength=0.02.mp4",
    decode_chunk_size=8,
    seed=42,
)

pipe = PipelineSVDUpscale(upscaler="instruct_ir")
pipe(
    img,
    prompt="Correct the motion blur in this image so it is more clear.",
    output_path="./test/test_video/test_svd_upscale.instruct_ir.motion_bucket_id=127.noise_aug_strength=0.02.mp4",
    decode_chunk_size=8,
    seed=42,
)


from .model_stable_video_diffusion import SVD
from .model_stable_diffusion_2 import SD2
from .model_stable_diffusion_3 import SD3
from .model_stable_diffusion_upscaler import SDUpScaler
from .model_depth_anything_v2 import DepthAnythingV2
from .model_depth2image import Depth2Image
from .model_clip_interrogator import CLIPInterrogator
from .model_instruct_ir import InstructIR
from .model_controlnet_stable_diffusion_2 import ControlNetSD2
from .model_controlnet_stable_diffusion_3 import ControlNetSD3
from .model_flux_1_dev import Flux1Dev



from PIL import Image
from panna.pipeline import PipelineDepth2ImageV2


pipe = PipelineDepth2ImageV2()
img = Image.open("./test/sample_image.png")
pipe(img, output_path="./test/test_image/test_depth2image_v2.png", prompt="a black cat", negative_prompt="bad, deformed, ugly, bad anatomy", seed=42)



"""Model class for stable depth2image."""
from typing import Optional, List
import torch
from diffusers import StableDiffusionDepth2ImgPipeline
from PIL.Image import Image
from .util import get_generator, clear_cache, get_logger

logger = get_logger(__name__)


class Depth2Image:

    base_model: StableDiffusionDepth2ImgPipeline

    def __init__(self,
                 base_model_id: str = "stabilityai/stable-diffusion-2-depth",
                 variant: Optional[str] = "fp16",
                 torch_dtype: Optional[torch.dtype] = torch.float16,
                 device_map: str = "balanced",
                 low_cpu_mem_usage: bool = True):
        self.base_model = StableDiffusionDepth2ImgPipeline.from_pretrained(
            base_model_id, use_safetensors=True, variant=variant, torch_dtype=torch_dtype, device_map=device_map, low_cpu_mem_usage=low_cpu_mem_usage
        )

    def text2image(self,
                   image: List[Image],
                   prompt: List[str],
                   depth_maps: Optional[List[torch.Tensor]] = None,
                   batch_size: Optional[int] = None,
                   negative_prompt: Optional[List[str]] = None,
                   guidance_scale: float = 7.5,
                   num_inference_steps: int = 50,
                   num_images_per_prompt: int = 1,
                   seed: Optional[int] = None) -> List[Image]:
        """Generate image from text.

        :param image:
        :param prompt:
        :param depth_maps: Depth prediction to be used as additional conditioning for the image generation process. If
            not defined, it automatically predicts the depth with self.depth_estimator.
        :param batch_size:
        :param negative_prompt: eg. "bad, deformed, ugly, bad anatomy"
        :param guidance_scale:
        :param num_inference_steps: Define how many steps and what % of steps to be run on each expert (80/20) here.
        :param num_images_per_prompt:
        :param seed:
        :return:
        """
        assert len(image) == len(prompt), f"{len(image)} != {len(prompt)}"
        if negative_prompt is not None:
            assert len(negative_prompt) == len(prompt), f"{len(negative_prompt)} != {len(prompt)}"
        if depth_maps is not None:
            assert len(depth_maps) == len(prompt), f"{len(depth_maps)} != {len(prompt)}"
        batch_size = len(prompt) if batch_size is None else batch_size
        idx = 0
        output_list = []
        while idx * batch_size < len(prompt):
            logger.info(f"[batch: {idx + 1}] generating...")
            start = idx * batch_size
            end = min((idx + 1) * batch_size, len(prompt))
            output_list += self.base_model(
                prompt=prompt[start:end],
                image=image[start:end],
                depth_map=None if depth_maps is None else torch.concat(depth_maps[start:end]),
                negative_prompt=None if negative_prompt is None else negative_prompt[start:end],
                guidance_scale=guidance_scale,
                num_inference_steps=num_inference_steps,
                num_images_per_prompt=num_images_per_prompt,
                generator=get_generator(seed)
            ).images
            idx += 1
            clear_cache()
        return output_list

    @staticmethod
    def export(data: Image, output_path: str, file_format: str = "png") -> None:
        data.save(output_path, file_format)



"""Model class for FLUX 1 dev."""
from typing import Optional, List
import torch
from diffusers import FluxPipeline
from PIL.Image import Image
from .util import get_generator, clear_cache, get_logger

logger = get_logger(__name__)


class Flux1Dev:

    base_model: FluxPipeline

    def __init__(self,
                 base_model_id: str = "black-forest-labs/FLUX.1-dev",
                 torch_dtype: torch.dtype = torch.float16,
                 device_map: str = "balanced",
                 low_cpu_mem_usage: bool = True):
        self.base_model = FluxPipeline.from_pretrained(
            base_model_id, use_safetensors=True, torch_dtype=torch_dtype, device_map=device_map, low_cpu_mem_usage=low_cpu_mem_usage
        )

    def text2image(self,
                   prompt: List[str],
                   batch_size: Optional[int] = None,
                   negative_prompt: Optional[List[str]] = None,
                   guidance_scale: float = 3.5,
                   num_inference_steps: int = 50,
                   num_images_per_prompt: int = 1,
                   height: Optional[int] = None,
                   width: Optional[int] = None,
                   seed: Optional[int] = None) -> List[Image]:
        """Generate image from text.

        :param prompt:
        :param batch_size:
        :param negative_prompt:
        :param guidance_scale:
        :param num_inference_steps:
        :param num_images_per_prompt:
        :param height:
        :param width:
        :param seed:
        :return:
        """
        if negative_prompt is not None:
            assert len(negative_prompt) == len(prompt), f"{len(negative_prompt)} != {len(prompt)}"
        batch_size = len(prompt) if batch_size is None else batch_size
        idx = 0
        output_list = []
        while idx * batch_size < len(prompt):
            logger.info(f"[batch: {idx + 1}] generating...")
            start = idx * batch_size
            end = min((idx + 1) * batch_size, len(prompt))
            output_list += self.base_model(
                prompt=prompt[start:end],
                negative_prompt=negative_prompt if negative_prompt is None else negative_prompt[start:end],
                guidance_scale=guidance_scale,
                num_images_per_prompt=num_images_per_prompt,
                num_inference_steps=num_inference_steps,
                height=height,
                width=width,
                generator=get_generator(seed)
            ).images
            idx += 1
            clear_cache()
        return output_list

    @staticmethod
    def export(data: Image, output_path: str, file_format: str = "png") -> None:
        data.save(output_path, file_format)



from panna import ControlNetSD2
from diffusers.utils import load_image

# test canny
model = ControlNetSD2(condition_type="canny")
prompt = "aerial view, a futuristic research complex in a bright foggy jungle, hard lighting"
negative_prompt = 'low quality, bad quality, sketches'
image = load_image("https://huggingface.co/datasets/hf-internal-testing/diffusers-images/resolve/main/sd_controlnet/hf-logo.png")
output = model.text2image([prompt], negative_prompt=[negative_prompt], image=[image])
model.export(output[0], "./test/test_image/test_controlnet_stable_diffusion_2.canny.png")


# test depth
model = ControlNetSD2(condition_type="depth")
prompt = "pixel-art margot robbie as barbie, in a coupé . low-res, blocky, pixel art style, 8-bit graphics"
negative_prompt = "sloppy, messy, blurry, noisy, highly detailed, ultra textured, photo, realistic"
image = load_image("https://media.vogue.fr/photos/62bf04b69a57673c725432f3/3:2/w_1793,h_1195,c_limit/rev-1-Barbie-InstaVert_High_Res_JPEG.jpeg")
output = model.text2image([prompt], negative_prompt=[negative_prompt], image=[image])
model.export(output[0], "./test/test_image/test_controlnet_stable_diffusion_2.depth.png")



"""Model class for stable diffusion2."""
from typing import Optional, Dict, List, Any
import torch
from diffusers import DiffusionPipeline, StableDiffusionXLPipeline
from PIL.Image import Image
from .util import get_generator, clear_cache, get_logger

logger = get_logger(__name__)


class SD2:

    config: Dict[str, Any]
    base_model_id: str
    base_model: StableDiffusionXLPipeline
    refiner_model: Optional[StableDiffusionXLPipeline]

    def __init__(self,
                 use_refiner: bool = True,
                 base_model_id: str = "stabilityai/stable-diffusion-xl-base-1.0",
                 refiner_model_id: str = "stabilityai/stable-diffusion-xl-refiner-1.0",
                 variant: str = "fp16",
                 torch_dtype: torch.dtype = torch.float16,
                 device_map: str = "balanced",
                 low_cpu_mem_usage: bool = True):
        config = dict(use_safetensors=True, variant=variant, torch_dtype=torch_dtype, device_map=device_map, low_cpu_mem_usage=low_cpu_mem_usage)
        self.base_model = StableDiffusionXLPipeline.from_pretrained(base_model_id, **config)
        self.refiner_model = None
        if use_refiner:
            self.refiner_model = DiffusionPipeline.from_pretrained(refiner_model_id, text_encoder_2=self.base_model.text_encoder_2, vae=self.base_model.vae, **config)

    def text2image(self,
                   prompt: List[str],
                   batch_size: Optional[int] = None,
                   negative_prompt: Optional[List[str]] = None,
                   guidance_scale: float = 7.5,
                   num_inference_steps: int = 40,
                   num_images_per_prompt: int = 1,
                   high_noise_frac: float = 0.8,
                   height: Optional[int] = None,
                   width: Optional[int] = None,
                   seed: Optional[int] = None) -> List[Image]:
        """Generate image from text.

        :param prompt:
        :param batch_size:
        :param negative_prompt:
        :param guidance_scale:
        :param num_inference_steps: Define how many steps and what % of steps to be run on each expert (80/20) here.
        :param num_images_per_prompt:
        :param high_noise_frac: Define how many steps and what % of steps to be run on refiner.
        :param height:
        :param width:
        :param seed:
        :return: List of images.
        """
        if negative_prompt is not None:
            assert len(negative_prompt) == len(prompt), f"{len(negative_prompt)} != {len(prompt)}"
        batch_size = len(prompt) if batch_size is None else batch_size
        idx = 0
        output_list = []
        while idx * batch_size < len(prompt):
            logger.info(f"[batch: {idx + 1}] generating...")
            start = idx * batch_size
            end = min((idx + 1) * batch_size, len(prompt))
            if self.refiner_model:
                output_list += self._text2image_refiner(
                    prompt=prompt[start:end],
                    negative_prompt=negative_prompt if negative_prompt is None else negative_prompt[start:end],
                    guidance_scale=guidance_scale,
                    num_inference_steps=num_inference_steps,
                    num_images_per_prompt=num_images_per_prompt,
                    high_noise_frac=high_noise_frac,
                    height=height,
                    width=width,
                    seed=seed
                )
            else:
                output_list += self._text2image(
                    prompt=prompt[start:end],
                    negative_prompt=negative_prompt if negative_prompt is None else negative_prompt[start:end],
                    guidance_scale=guidance_scale,
                    num_inference_steps=num_inference_steps,
                    num_images_per_prompt=num_images_per_prompt,
                    height=height,
                    width=width,
                    seed=seed
                )
            idx += 1
            clear_cache()
        return output_list

    def _text2image_refiner(self,
                            prompt: List[str],
                            negative_prompt: Optional[List[str]],
                            guidance_scale: float,
                            num_inference_steps: int,
                            num_images_per_prompt: int,
                            high_noise_frac: float,
                            height: Optional[int],
                            width: Optional[int],
                            seed: Optional[int] = None) -> List[Image]:
        assert self.refiner_model
        image = self.base_model(
            prompt=prompt,
            negative_prompt=negative_prompt,
            guidance_scale=guidance_scale,
            num_inference_steps=num_inference_steps,
            num_images_per_prompt=num_images_per_prompt,
            output_type="latent",
            denoising_end=high_noise_frac,
            height=height,
            width=width,
            generator=get_generator(seed)
        ).images
        return self.refiner_model(
            image=image,
            prompt=prompt,
            negative_prompt=negative_prompt,
            num_inference_steps=num_inference_steps,
            denoising_start=high_noise_frac,
            height=height,
            width=width,
            generator=get_generator(seed)
        ).images

    def _text2image(self,
                    prompt: List[str],
                    negative_prompt: Optional[List[str]],
                    guidance_scale: float,
                    num_inference_steps: int,
                    num_images_per_prompt: int,
                    height: Optional[int],
                    width: Optional[int],
                    seed: Optional[int] = None) -> List[Image]:
        return self.base_model(
            prompt=prompt,
            negative_prompt=negative_prompt,
            guidance_scale=guidance_scale,
            num_inference_steps=num_inference_steps,
            num_images_per_prompt=num_images_per_prompt,
            height=height,
            width=width,
            generator=get_generator(seed)
        ).images

    @staticmethod
    def export(data: Image, output_path: str, file_format: str = "png") -> None:
        data.save(output_path, file_format)



import random
import gc
import logging
from typing import Optional

import numpy as np
from PIL import Image
from torch import Generator
from torch import cuda


def get_generator(seed: Optional[int] = None) -> Generator:
    if seed:
        return Generator().manual_seed(seed)
    max_seed = np.iinfo(np.int32).max
    return Generator().manual_seed(random.randint(0, max_seed))


def clear_cache() -> None:
    gc.collect()
    cuda.empty_cache()


def get_logger(name: str) -> logging.Logger:
    logger = logging.getLogger(name)
    logging.basicConfig(
        format="%(asctime)s - %(levelname)s - %(name)s - %(message)s",
        datefmt="%m/%d/%Y %H:%M:%S",
        level=logging.INFO,
    )
    return logger


def resize_image(image: Image.Image, width: int, height: int) -> Image.Image:
    # Calculate aspect ratios
    target_aspect = width / height  # Aspect ratio of the desired size
    image_aspect = image.width / image.height  # Aspect ratio of the original image
    if image_aspect > target_aspect:  # Resize the image to match the target height, maintaining aspect ratio
        new_width = int(height * image_aspect)
        resized_image = image.resize((new_width, height), Image.LANCZOS)
        left, top, right, bottom = (new_width - width) / 2, 0, (new_width + width) / 2, height
    else:  # Resize the image to match the target width, maintaining aspect ratio
        new_height = int(width / image_aspect)
        resized_image = image.resize((width, new_height), Image.LANCZOS)
        # Calculate coordinates for cropping
        left, top, right, bottom = 0, (new_height - height) / 2, width, (new_height + height) / 2
    return resized_image.crop((left, top, right, bottom))



from PIL import Image
from panna import DepthAnythingV2

model = DepthAnythingV2()
img = Image.open("./test/sample_image.png")
output = model.image2depth([img], batch_size=1)
model.export(output[0], "./test/test_image/test_depth_anything.png")



import os
from PIL import Image
from panna import SVD
from diffusers.utils import load_image

model = SVD()
img = Image.open("./test/sample_image.png")
os.makedirs("./test/test_video", exist_ok=True)

data = model.image2video([img], seed=42, decode_chunk_size=8, noise_aug_strength=0.01)
model.export(data[0], "./test/test_video/test_svd.1.motion_bucket_id=127.noise_aug_strength=0.01.mp4")

img = load_image("https://huggingface.co/spaces/multimodalart/stable-video-diffusion/resolve/main/images/wink_meme.png")
data = model.image2video([img], seed=42, decode_chunk_size=8, noise_aug_strength=0.01)
model.export(data[0], "./test/test_video/test_svd.2.motion_bucket_id=127.noise_aug_strength=0.01.mp4")



import os
from panna import SDUpScaler
from diffusers.utils import load_image


model = SDUpScaler()
sample_image = load_image("https://huggingface.co/spaces/multimodalart/stable-video-diffusion/resolve/main/images/wink_meme.png")
os.makedirs("./test/test_image", exist_ok=True)

output = model.image2image([sample_image], prompt=["a lady blinking"], reshape_method="best")
model.export(output[0], "./test/test_image/test_sd_upscaler.prompt.reshape_method=best.png")

output = model.image2image([sample_image], prompt=["a lady blinking"], reshape_method="downscale")
model.export(output[0], "./test/test_image/test_sd_upscaler.prompt.reshape_method=downscale.png")



"""Model class for controlnet."""
from typing import Optional, List, Callable
import torch
import numpy as np
from cv2 import Canny
from diffusers import StableDiffusionXLControlNetPipeline, ControlNetModel, AutoencoderKL
from PIL.Image import Image, fromarray
from .util import get_generator, clear_cache, get_logger
from .model_depth_anything_v2 import DepthAnythingV2

logger = get_logger(__name__)


def get_canny_edge(image: Image) -> Image:
    image = Canny(np.array(image), 100, 200)
    image = image[:, :, None]
    image = np.concatenate([image, image, image], axis=2)
    return fromarray(image)


class ControlNetSD2:

    base_model: ControlNetModel
    get_condition: Callable[[Image], Image]

    def __init__(self,
                 base_model_id: str = "stabilityai/stable-diffusion-xl-base-1.0",
                 vae_model_id: str = "madebyollin/sdxl-vae-fp16-fix",
                 condition_type: str = "canny",
                 variant: str = "fp16",
                 torch_dtype: torch.dtype = torch.float16,
                 device_map: str = "balanced",
                 low_cpu_mem_usage: bool = True):
        vae = AutoencoderKL.from_pretrained(
            vae_model_id, torch_dtype=torch_dtype, device_map=device_map, low_cpu_mem_usage=low_cpu_mem_usage
        )
        if condition_type == "canny":
            controlnet = ControlNetModel.from_pretrained(
                "diffusers/controlnet-canny-sdxl-1.0", use_safetensors=True, torch_dtype=torch_dtype, variant=variant, low_cpu_mem_usage=low_cpu_mem_usage
            )
            self.get_condition = get_canny_edge
        elif condition_type == "depth":
            controlnet = ControlNetModel.from_pretrained(
                "diffusers/controlnet-zoe-depth-sdxl-1.0", torch_dtype=torch_dtype, low_cpu_mem_usage=low_cpu_mem_usage
            )
            depth = DepthAnythingV2()
            self.get_condition = lambda x: depth.image2depth([x])[0]
        else:
            raise ValueError(f"unknown condition: {condition_type}")
        if torch.cuda.is_available():
            controlnet = controlnet.cuda()
        self.base_model = StableDiffusionXLControlNetPipeline.from_pretrained(
            base_model_id, controlnet=controlnet, vae=vae, use_safetensors=True, device_map=device_map, torch_dtype=torch_dtype, variant=variant, low_cpu_mem_usage=low_cpu_mem_usage
        )

    def text2image(self,
                   prompt: List[str],
                   image: List[Image],
                   controlnet_conditioning_scale: float = 0.5,
                   batch_size: Optional[int] = None,
                   negative_prompt: Optional[List[str]] = None,
                   guidance_scale: float = 7.5,
                   num_inference_steps: int = 40,
                   num_images_per_prompt: int = 1,
                   height: Optional[int] = None,
                   width: Optional[int] = None,
                   seed: Optional[int] = None) -> List[Image]:
        """Generate image from text prompt with conditioning.

        :param prompt:
        :param image:
        :param controlnet_conditioning_scale:
        :param batch_size:
        :param negative_prompt:
        :param guidance_scale:
        :param num_inference_steps:
        :param num_images_per_prompt:
        :param height:
        :param width:
        :param seed:
        :return:
        """
        assert len(prompt) == len(image), f"{len(prompt)} != {len(image)}"
        if negative_prompt is not None:
            assert len(negative_prompt) == len(prompt), f"{len(negative_prompt)} != {len(prompt)}"
        batch_size = len(prompt) if batch_size is None else batch_size
        idx = 0
        output_list = []
        logger.info("generate condition")
        condition = []
        for x in image:
            condition.append(self.get_condition(x))
        logger.info("generate image")
        while idx * batch_size < len(prompt):
            logger.info(f"[batch: {idx + 1}] generating...")
            start = idx * batch_size
            end = min((idx + 1) * batch_size, len(prompt))
            output_list += self.base_model(
                prompt=prompt[start:end],
                image=condition[start:end],
                controlnet_conditioning_scale=controlnet_conditioning_scale,
                negative_prompt=negative_prompt if negative_prompt is None else negative_prompt[start:end],
                guidance_scale=guidance_scale,
                num_images_per_prompt=num_images_per_prompt,
                num_inference_steps=num_inference_steps,
                height=height,
                width=width,
                generator=get_generator(seed)
            ).images
            idx += 1
            clear_cache()
        return output_list

    @staticmethod
    def export(data: Image, output_path: str, file_format: str = "png") -> None:
        data.save(output_path, file_format)



from PIL import Image
from panna.pipeline import PipelineLEditsPP
from diffusers.utils import load_image


img = Image.open("./test/sample_image.png")
pipe = PipelineLEditsPP()
pipe(img,
     output_path="./test/test_image/test_ledits_pp.1.png",
     edit_prompt=["lion", "cat"],
     reverse_editing_direction=[True, False],
     seed=42)
pipe(img,
     output_path="./test/test_image/test_ledits_pp.2.png",
     edit_prompt=["lion", "cat"],
     edit_style=["object", "object"],
     reverse_editing_direction=[True, False],
     seed=42)

img = load_image("https://huggingface.co/spaces/multimodalart/stable-video-diffusion/resolve/main/images/wink_meme.png")
pipe(img,
     output_path="./test/test_image/test_ledits_pp.3.png",
     edit_prompt=["woman", "man"],
     edit_style=["face", "face"],
     reverse_editing_direction=[True, False],
     seed=42)



"""Model class for InstructIR. https://huggingface.co/spaces/marcosv/InstructIR"""
from typing import List, Optional
import torch
import numpy as np
from PIL import Image
from huggingface_hub import hf_hub_download

from .modules_instruct_ir.instructir import create_model
from .modules_instruct_ir.language_models import LMHead, LanguageModel
from .util import clear_cache, get_logger

logger = get_logger(__name__)


class InstructIR:

    def __init__(self):
        self.device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
        model_path = hf_hub_download(repo_id="marcosv/InstructIR", filename="im_instructir-7d.pt")
        self.model = create_model()
        self.model = self.model.to(self.device)
        self.model.load_state_dict(torch.load(model_path, map_location="cpu"), strict=True)
        model_path_lm = hf_hub_download(repo_id="marcosv/InstructIR", filename="lm_instructir-7d.pt")
        self.lm_head = LMHead()
        self.lm_head = self.lm_head.to(self.device)
        self.lm_head.load_state_dict(torch.load(model_path_lm, map_location="cpu"), strict=True)
        self.language_model = LanguageModel()
        self.language_model.model = self.language_model.model.to(self.device)

    def preprocess(self, image: Image.Image) -> torch.Tensor:
        image = np.array(image)
        image = image / 255.
        image = image.astype(np.float32)
        return torch.tensor(image).permute(2, 0, 1).unsqueeze(0).to(self.device)

    def image2image(self,
                    image: List[Image.Image],
                    prompt: Optional[List[str]] = None) -> List[Image.Image]:
        """Generate high resolution image from low resolution image.

        :param image:
        :param prompt:
        :return:
        """
        prompt = ["Correct the motion blur in this image so it is more clear"] * len(image) if prompt is None else prompt
        assert len(prompt) == len(image), f"{len(prompt)} != {len(image)}"
        output_list = []
        for i in image:
            with torch.no_grad():
                y = self.preprocess(i)
                lm_embd = self.language_model(prompt).to(self.device)
                text_embd, deg_pred = self.lm_head(lm_embd)
                x_hat = self.model(y, text_embd)
                restored_img = x_hat.squeeze().permute(1, 2, 0).clamp_(0, 1).cpu().detach().numpy()
                restored_img = np.clip(restored_img, 0.0, 1.)
                restored_img = (restored_img * 255.0).round().astype(np.uint8)
            output_list.append(Image.fromarray(restored_img))
            clear_cache()
        return output_list

    @staticmethod
    def export(data: Image, output_path: str, file_format: str = "png") -> None:
        data.save(output_path, file_format)



from setuptools import setup, find_packages

with open('README.md', 'r', encoding="utf-8") as f:
    readme = f.read()

version = '0.0.10'
setup(
    name='panna',
    packages=find_packages(exclude=["hf_space", "tests"]),
    version=version,
    license='MIT',
    description='PANNA',
    url='https://github.com/asahi417',
    keywords=['machine-learning', 'computational-art'],
    long_description=readme,
    long_description_content_type="text/markdown",
    author='Asahi Ushio',
    author_email='asahi1992ushio@gmail.com',
    classifiers=[
        'Development Status :: 4 - Beta',       # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',      # Define that your audience are developers
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    ],
    include_package_data=True,
    test_suite='tests',
    install_requires=[
        "torch>=2.0",
        "datasets",
        "transformers",
        "diffusers>=0.19.0",
        "invisible_watermark",
        "accelerate",
        "safetensors",
        "sentencepiece",
        "protobuf",
        "clip-interrogator",
        "accelerate",
        "opencv-python",
        "controlnet_aux"
    ],
    python_requires='>=3.8',
    entry_points={
        'console_scripts': [],
    }
)


"""Model class for stable diffusion upscaler."""
from typing import Optional, List
import torch
from diffusers import StableDiffusionUpscalePipeline
from PIL.Image import Image
from .util import clear_cache, get_logger, resize_image

logger = get_logger(__name__)


class SDUpScaler:

    base_model: StableDiffusionUpscalePipeline
    height: int = 128
    width: int = 128

    def __init__(self,
                 base_model_id: str = "stabilityai/stable-diffusion-x4-upscaler",
                 variant: str = "fp16",
                 torch_dtype: torch.dtype = torch.float16,
                 device_map: str = "balanced",
                 low_cpu_mem_usage: bool = True):
        self.base_model = StableDiffusionUpscalePipeline.from_pretrained(
            base_model_id, use_safetensors=True, variant=variant, torch_dtype=torch_dtype, device_map=device_map, low_cpu_mem_usage=low_cpu_mem_usage
        )

    def image2image(self,
                    image: List[Image],
                    prompt: Optional[List[str]] = None,
                    reshape_method: Optional[str] = "downscale",
                    upscale_factor: int = 4,
                    batch_size: Optional[int] = None) -> List[Image]:
        """Generate high resolution image from low resolution image.

        :param image:
        :param prompt:
        :param reshape_method:
        :param upscale_factor:
        :param batch_size:
        :return:
        """

        def downscale_image(x: Image) -> Image:
            return resize_image(x, width=int(x.width / upscale_factor), height=int(x.height / upscale_factor))

        prompt = [""] * len(image) if prompt is None else prompt
        assert len(prompt) == len(image), f"{len(prompt)} != {len(image)}"
        batch_size = len(prompt) if batch_size is None else batch_size
        idx = 0
        output_list = []
        while idx * batch_size < len(prompt):
            logger.info(f"[batch: {idx + 1}] generating...")
            start = idx * batch_size
            end = min((idx + 1) * batch_size, len(prompt))
            batch = image[start:end]
            if reshape_method == "best":
                batch = [resize_image(i, width=self.width, height=self.height) for i in image]
            elif reshape_method == "downscale":
                batch = [downscale_image(i) for i in image]
            elif reshape_method is not None:
                raise ValueError(f"unknown reshape method: {reshape_method}")
            output_list += self.base_model(image=batch, prompt=prompt[start:end]).images
            idx += 1
            clear_cache()
        return output_list

    @staticmethod
    def export(data: Image, output_path: str, file_format: str = "png") -> None:
        data.save(output_path, file_format)



import torch
import torch.nn as nn
import torch.nn.functional as F

from .nafnet import NAFBlock


class ICB(nn.Module):
    def __init__(self, feature_dim, text_dim=768):
        super(ICB, self).__init__()
        self.fc = nn.Linear(text_dim, feature_dim)
        self.block = NAFBlock(feature_dim)
        self.beta = nn.Parameter(torch.zeros((1, feature_dim, 1, 1)), requires_grad=True)
        self.gamma = nn.Parameter(torch.zeros((1, feature_dim, 1, 1)), requires_grad=True)

    def forward(self, x, text_embedding):
        gating_factors = torch.sigmoid(self.fc(text_embedding))
        gating_factors = gating_factors.unsqueeze(-1).unsqueeze(-1)
        f = x * self.gamma + self.beta  # 1) learned feature scaling/modulation
        f = f * gating_factors          # 2) (soft) feature routing based on text
        f = self.block(f)               # 3) block feature enhancement
        return f + x


class InstructIR(nn.Module):
    def __init__(self, img_channel=3, width=16, middle_blk_num=1, enc_blk_nums=[], dec_blk_nums=[], txtdim=768):
        super().__init__()
        self.intro = nn.Conv2d(in_channels=img_channel, out_channels=width, kernel_size=3, padding=1, stride=1, groups=1, bias=True)
        self.ending = nn.Conv2d(in_channels=width, out_channels=img_channel, kernel_size=3, padding=1, stride=1, groups=1, bias=True)
        self.encoders = nn.ModuleList()
        self.decoders = nn.ModuleList()
        self.middle_blks = nn.ModuleList()
        self.ups = nn.ModuleList()
        self.downs = nn.ModuleList()
        self.enc_cond = nn.ModuleList()
        self.dec_cond = nn.ModuleList()
        chan = width
        for num in enc_blk_nums:
            self.encoders.append(nn.Sequential(*[NAFBlock(chan) for _ in range(num)]))
            self.enc_cond.append(ICB(chan, txtdim))
            self.downs.append(nn.Conv2d(chan, 2*chan, 2, 2))
            chan = chan * 2
        self.middle_blks = nn.Sequential(*[NAFBlock(chan) for _ in range(middle_blk_num)])
        for num in dec_blk_nums:
            self.ups.append(nn.Sequential(nn.Conv2d(chan, chan * 2, 1, bias=False), nn.PixelShuffle(2)))
            chan = chan // 2
            self.decoders.append(nn.Sequential(*[NAFBlock(chan) for _ in range(num)]))
            self.dec_cond.append(ICB(chan, txtdim))
        self.padder_size = 2 ** len(self.encoders)

    def forward(self, inp, txtembd):
        B, C, H, W = inp.shape
        inp = self.check_image_size(inp)
        x = self.intro(inp)
        encs = []
        for encoder, enc_mod, down in zip(self.encoders, self.enc_cond, self.downs):
            x = encoder(x)
            x = enc_mod(x, txtembd)
            encs.append(x)
            x = down(x)
        x = self.middle_blks(x)
        for decoder, up, enc_skip, dec_mod in zip(self.decoders, self.ups, encs[::-1], self.dec_cond):
            x = up(x)
            x = x + enc_skip
            x = decoder(x)
            x = dec_mod(x, txtembd)
        x = self.ending(x)
        x = x + inp
        return x[:, :, :H, :W]

    def check_image_size(self, x):
        _, _, h, w = x.size()
        mod_pad_h = (self.padder_size - h % self.padder_size) % self.padder_size
        mod_pad_w = (self.padder_size - w % self.padder_size) % self.padder_size
        x = F.pad(x, (0, mod_pad_w, 0, mod_pad_h))
        return x


def create_model():
    return InstructIR(
        img_channel=3,
        width=32,
        middle_blk_num=4,
        enc_blk_nums=[2, 2, 4, 8],
        dec_blk_nums=[2, 2, 2, 2],
        txtdim=256
    )



import torch
from typing import Optional
from PIL.Image import Image
from panna import Depth2Image, DepthAnythingV2
from panna.util import get_logger

logger = get_logger(__name__)


class PipelineDepth2ImageV2:

    def __init__(self,
                 variant: Optional[str] = "fp16",
                 torch_dtype: Optional[torch.dtype] = torch.float16,
                 device_map: str = "balanced",
                 low_cpu_mem_usage: bool = True):
        self.depth2image = Depth2Image(
            variant=variant, torch_dtype=torch_dtype, device_map=device_map, low_cpu_mem_usage=low_cpu_mem_usage
        )
        self.depth_anything = DepthAnythingV2()

    def __call__(self,
                 image: Image,
                 prompt: str,
                 output_path: Optional[str] = None,
                 negative_prompt: Optional[str] = None,
                 guidance_scale: float = 7.5,
                 num_inference_steps: int = 50,
                 seed: Optional[int] = None) -> Image:
        logger.info("run depth anything v2")
        depth = self.depth_anything.image2depth([image], return_tensor=True)
        logger.info("run depth2image")
        image = self.depth2image.text2image(
            [image],
            prompt=[prompt],
            depth_maps=depth,
            negative_prompt=[negative_prompt],
            guidance_scale=guidance_scale,
            num_inference_steps=num_inference_steps,
            seed=seed
        )[0]
        if output_path:
            self.depth2image.export(image, output_path)
        return image
